from __future__ import absolute_import, unicode_literals

HOSTNAME_TO_JOURNAL_MAP = {
		'molbiolcell.org': 'Mol Biol Cell',
		'biomedgerontology.oxfordjournals.org': 'J Gerontol A Biol Sci Med Sci',
		'jrr.oxfordjournals.org': 'Radiat Res',
		'ajpendo.physiology.org': 'Am J Physiol Endocrinol Metab',
		'cancerdiscovery.aacrjournals.org': 'Cancer Discov',
		'ajpheart.physiology.org': 'Am J Physiol Heart Circ Physiol',
		'jnnp.bmj.com': 'J Neurol Neurosurg Psychiatry',
		'jvi.asm.org': 'J Virol',
		'jisakos.bmj.com': 'J ISAKOS',
		'jme.bmj.com': 'J Med Ethics',
		'adc.bmj.com': 'Arch Dis Child',
		'atvb.ahajournals.org': 'Arterioscler Thromb Vasc Biol',
		'erc.endocrinology-journals.org': 'Endocr Relat Cancer',
		'qjmed.oxfordjournals.org': 'QJM',
		'ajcp.ascpjournals.org': 'Am J Clin Pathol',
		'jid.oxfordjournals.org': 'J Infect Dis',
		'jfprhc.bmj.com': 'J Fam Plann Reprod Health Care',
		'ang.sagepub.com': 'Angiology',
		'jcn.sagepub.com': 'J Child Neurol',
		'iovs.org': 'Invest Ophthalmol Vis Sci',
		'iovs.org': 'IOVS',
		'ndt.oxfordjournals.org': 'Nephrol Dial Transplant',
		'plantcell.org': 'Plant Cell',
		'erj.ersjournals.com': 'Eur Respir J',
		'injuryprevention.bmj.com': 'Inj Prev',
		'genesdev.cshlp.org': 'Genes Dev',
		'vetrecordopen.bmj.com': 'Vet Rec Open',
		'vetrecordcasereports.bmj.com': 'Vet Rec Case Rep',
		'fasebj.org': 'FASEB J',
		'ajh.oxfordjournals.org': 'Am J Hypertens',
		'jco.ascopubs.org': 'J Clin Oncol',
		'mct.aacrjournals.org': 'Mol Cancer Ther',
		'hyper.ahajournals.org': 'Hypertension',
		'cat.sagepub.com': 'Clin Appl Thromb Hemost',
		'rheumatology.oxfordjournals.org': 'Rheumatology (Oxford)',
		'jem.rupress.org': 'J Exp Med',
		'jmg.bmj.com': 'J Med Genet',
		'ebmh.bmj.com': 'Evid Based Mental Health',
		'ajpgi.physiology.org': 'Am J Physiol Gastrointest Liver Physiol',
		'care.diabetesjournals.org': 'Diabetes Care',
		'dmd.aspetjournals.org': 'Drug Metab Dispos',
		'ard.bmj.com': 'Ann Rheum Dis',
		'jap.physiology.org': 'J Appl Physiol',
		'aac.asm.org': 'Antimicrob Agents Chemother',
		'cfp.ca': 'Can Fam Physician',
		'clincancerres.aacrjournals.org': 'Clin Cancer Res',
		'bmjopenrespres.bmj.com': 'BMJ Open Resp Res',
		'diabetes.diabetesjournals.org': 'Diabetes',
		'jramc.bmj.com': 'J R Army Med Corps',
		'openheart.bmj.com': 'Open Heart',
		'ajpcell.physiology.org': 'Am J Physiol Cell Physiol',
		'physiologyonline.physiology.org': 'Physiology (Bethesda)',
		'gut.bmj.com': 'Gut',
		'mct.aacrjournals.org': 'Mol Canc Therapeut',
		'cpj.sagepub.com': 'Clin Pediatr',
		'humrep.oxfordjournals.org': 'Hum Reprod',
		'pcp.oxfordjournals.org': 'Plant Cell Physiol',
		'peds.oxfordjournals.org': 'Protein Eng',
		'jbjs.org': 'J Bone Joint Surg Am',
		'bjo.bmj.com': 'BMC Ophthalmol',
		'ajplung.physiology.org': 'Am J Physiol Lung Cell Mol Physiol',
		'jdr.sagepub.com': 'J Dent Res',
		'physreports.physiology.org': 'Physiol Rep',
		'physrev.physiology.org': 'Physiol Rev',
		'asm.sagepub.com': 'Assessment',
		'jac.oxfordjournals.org': 'J Antimicrob Chemother',
		'clinchem.org': 'Clin Chem',
		'jah.sagepub.com': 'J Aging Health',
		'cid.oxfordjournals.org': 'Clin Infect Dis',
		'msphere.asm.org': 'mSphere',
		'molehr.oxfordjournals.org': 'Mol Hum Reprod',
		'lup.sagepub.com': 'Lupus',
		'mh.bmj.com': 'Med Humanities',
		'aim.bmj.com': 'Acupunct Med',
		'ec.asm.org': 'Eukaryot Cell',
		'bmjopensem.bmj.com': 'BMJ Open Sport Exerc Med',
		'ajprenal.physiology.org': 'Am J Physiol Renal Physiol',
		'acb.sagepub.com': 'Ann Clin Biochem',
		'qir.bmj.com': 'BMJ Qual Improv Report',
		'stroke.ahajournals.org': 'Stroke',
		'emboj.embopress.org': 'EMBO J',
		'jcm.asm.org': 'J Clin Microbiol',
		'ajpregu.physiology.org': 'Am J Physiol Regul Integr Comp Physiol',
		'circep.ahajournals.org': 'Circ Arrhythm Electrophysiol',
		'drc.bmj.com': 'BMJ Open Diabetes Res Care',
		'jim.bmj.com': 'J Investig Med',
		'jnci.oxfordjournals.org': 'J Natl Cancer Inst',
		'bja.oxfordjournals.org': 'Br J Anaesth',
		'eje-online.org': 'Eur J Endocrinol',
		'ebn.bmj.com': 'Evid Based Nurs',
		'ar.iiarjournals.org': 'Anticancer Res',
		'iai.asm.org': 'Infect Immun',
		'inpractice.bmj.com': 'In Pract',
		'dtb.bmj.com': 'Drug Ther Bull',
		'eolj.bmj.com': 'End Life J',
		'pediatrics.aappublications.org': 'Pediatrics',
		'physiolgenomics.physiology.org': 'Physiol Genomics',
		'genomebiology.com': 'Genome Biol',
		'jimmunol.org': 'J Immunol',
		'fn.bmj.com': 'Arch Dis Child Fetal Neonatal Ed',
		'innovations.bmj.com': 'BMJ Innov',
		'heart.bmj.com': 'Heart',
		'emj.bmj.com': 'Emerg Med J',
		'ee.oxfordjournals.org': 'Environ Entomol',
		'mbio.asm.org': 'MBio',
		'mcb.asm.org': 'Mol Cell Biol',
		'pnas.org': 'Proc Natl Acad Sci USA',
		'aem.asm.org': 'Appl Environ Microbiol',
		'qualitysafety.bmj.com': 'BMJ Qual Saf',
		'stel.bmj.com': 'BMJ STEL',
		'jpet.aspetjournals.org': 'J Pharmacol Exp Ther',
		'jra.sagepub.com': 'J Renin Angiotensin Aldosterone Syst',
		'breast-cancer-research.com': 'Breast Cancer Res',
		'eurheartj.oxfordjournals.org': 'Eur Heart J',
		'genome.cshlp.org': 'Genome Res',
		'jb.asm.org': 'J Bacteriol',
		'jcb.rupress.org': 'J Cell Biol',
		'mutage.oxfordjournals.org': 'Mutagenesis',
		'haematologica.org': 'Haematologica',
		'jcp.bmj.com': 'J Clin Pathol',
		'ojrd.com': 'Orphanet J Rare Dis',
		'veterinaryrecord.bmj.com': 'Vet Rec',
		'femsle.oxfordjournals.org': 'FEMS Microbiol Lett',
		'jneurosci.org': 'J Neurosci',
		'jhl.sagepub.com': 'J Hum Lact',
		'oem.bmj.com': 'Occup Environ Med',
		'tobaccocontrol.bmj.com': 'Tob Control',
		'cebp.aacrjournals.org': 'Cancer Epidemiol Biomarkers Prev',
		'jme.endocrinology-journals.org': 'J Mol Endocrinol',
		'genomea.asm.org': 'Genome Announc',
		'spcare.bmj.com': 'BMJ Support Palliat Care',
		'pmj.bmj.com': 'Postgrad Med J',
		'molpharm.aspetjournals.org': 'Mol Pharmacol',
		'jbc.org': 'J Biol Chem',
		'peds.oxfordjournals.org': 'Protein Eng Des Sel',
		'sti.bmj.com': 'Sex Transm Infect',
		'jasn.asnjournals.org': 'J Am Soc Nephrol',
		'pn.bmj.com': 'Pract Neurol',
		'msystems.asm.org': 'mSystems',
		'thorax.bmj.com': 'Thorax',
		'fg.bmj.com': 'Frontline Gastroenterol',
		'cmr.asm.org': 'Clin Microbiol Rev',
		'circ.ahajournals.org': 'Circulation',
		'ebm.bmj.com': 'Evid Based Med',
		'carcin.oxfordjournals.org': 'Carcinogenesis',
		'lupus.bmj.com': 'Lupus Sci Med',
		'jlr.org': 'J Lipid Res',
		'jb.physiology.org': 'J Neurophysiol',
		'jrheum.org': 'J Rheumatol',
		'circres.ahajournals.org': 'Circ Res',
		'esmoopen.bmj.com': 'ESMO Open',
		'ejhp.bmj.com': 'Eur J Hosp Pharm',
		'neurology.org': 'Neurology',
		'ep.bmj.com': 'Arch Dis Child Educ Pract Ed',
		'cancerres.aacrjournals.org': 'Cancer Res',
		'bmjopen.bmj.com': 'BMJ Open',
		'annonc.oxfordjournals.org': 'Ann Oncol',
		'jn.nutrition.org': 'J Nutr',
		'cardiovascres.oxfordjournals.org': 'Cardiovasc Res',
		'heartasia.bmj.com': 'Heart Asia',
		'circgenetics.ahajournals.org': 'Circ Cardiovasc Genet',
		'cvi.asm.org': 'Clin Vaccine Immunol',
		'sciencemag.org': 'Science',
		'jnis.bmj.com': 'J Neurointerv Surg',
		'jcs.biologists.org': 'J Cell Sci',
		'hmg.oxfordjournals.org': 'Hum Mol Genet',
		'mmbr.asm.org': 'Microbiol Mol Biol Rev',
		'rmdopen.bmj.com': 'RMD Open',
		'nar.oxfordjournals.org': 'Nucleic Acids Res',
		'bjsm.bmj.com': 'Br J Sports Med',
		'jech.bmj.com': 'J Epidemiol Community Health',
		'brain.oxfordjournals.org': 'Brain',
		'cpj.sagepub.com': 'Clin Pediatr (Phila)',
		'jcem.endojournals.org': 'J Clin Endocrinol Metab',
		'joe.endocrinology-journals.org': 'J Endocrinol',
		'bmj.com': 'BMJ',
		'bloodjournal.org': 'Blood',
		'jmd.amjpathol.org': 'J Mol Diagn',
		'heartrhythmjournal.com': 'Heart Rhythm',
		'ajconline.org': 'Am J Cardiol',
		'cancergeneticsjournal.org': 'Cancer Genet Cytogenet',
		'epires-journal.com': 'Epilepsy Res',
		'amjmed.com': 'Am J Med',
		'nmd-journal.com': 'Neuromuscul Disord',
		'arcmedres.com': 'Arch Med Res',
		'jpeds.com': 'J Pediatr',
		'clineu-journal.com': 'Clin Neurol Neurosurg',
		'ijporlonline.com': 'Int J Pediatr Otorhinolaryngol',
		'mgmjournal.com': 'Mol Genet Metab',
		'ahjonline.com': 'Am Heart J',
		'cancerci.com': 'Cancer Cell Int',
		'internationaljournalofcardiology.com': 'Int J Cardiol',
		'akademiai.com': 'Orv Hetil',
		'placentajournal.org': 'Placenta',
		'prd-journal.com': 'Parkinsonism Relat Disord',
		'pedneur.com': 'Pediatr Neurol',
		'cancerletters.info': 'Cancer Lett',
		'neurobiologyofaging.org': 'Neurobiol Aging',
		'metabolismjournal.com': 'Metabolism',
		'thebonejournal.com': 'Bone',
		'thrombosisresearch.com': 'Thromb Res',
		'brainanddevelopment.com': 'Brain Dev',
		'jpurol.com': 'J Pediatr Urol',
		'metabolismjournal.com': 'Metab Clin Exp',
		'worldneurosurgery.org': 'Surg Neurol',
		'jmmc-online.com': 'J Mol Cell Cardiol',
		'gastrojournal.org': 'Gastroenterology',
		'exphem.org': 'Exp Hematol',
		'medical-hypotheses.com': 'Med Hypotheses',
		'atherosclerosis-journal.com': 'Atherosclerosis',
		'gynecologiconcology-online.net': 'Gynecol Oncol',
		'aaojournal.org': 'Ophthalmology',
		'biologicalpsychiatryjournal.com': 'Biol Psychiatry',
		'ejpn-journal.com': 'Eur J Paediatr Neurol',
		'jdsjournal.com': 'J Dermatol',
		'ajo.com': 'Am J Ophthalmol',
		'fertstert.org': 'Fertil Steril',
		'jns-journal.com': 'J Neurol Sci',
		'diabetesresearchclinicalpractice.com': 'Diabetes Res Clin Pract',
		'clinsci.org': 'Clin Sci',
		'biochemj.org': 'Biochem J',
		'sciencemag.org': 'Science',
		'stm.sciencemag.org': 'Sci Transl Med',
		'advances.sciencemag.org': 'Sci Adv',
		'stke.sciencemag.org': 'Sci Signal',
		'joponline.org': 'J Periodontol',
		'medicinabuenosaires.com': 'Medicina (B Aires)',
}
