from __future__ import absolute_import, unicode_literals

# cell journals
# OLD cell_format = 'http://download.cell.com{ja}/pdf/PII{pii}.pdf'
cell_format = 'http://www.cell.com{ja}/pdf/{pii}.pdf'
cell_journals = {
    'Am J Hum Genet': {'ja': '/AJHG'},
    'Biophys J': {'ja': '/biophysj'},
    'Cancer Cell': {'ja': '/cancer-cell'},
    'Cell': {'ja': ''},
    'Cell Host Microbe': {'ja': '/cell-host-microbe'},
    'Cell Metab': {'ja': '/cell-metabolism'},
    'Cell Stem Cell': {'ja': '/cell-stem-cell'},
    'Chem Biol': {'ja': '/chemistry-biology'},
    'Curr Biol': {'ja': '/current-biology'},
    'Dev Cell': {'ja': '/developmental-cell'},
    'Immunity': {'ja': '/immunity'},
    'Mol Cell': {'ja': '/molecular-cell'},
    'Neuron': {'ja': '/neuron'},
    'Structure': {'ja': '/structure'},
    'Trends Mol Med': {'ja': '/trends'},
}
